#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# pyDialer - "A simple and handy frontend for WvDial, fork of pyWvDial"
# Copyright (C) 2008 Giacomo Lozito
# Copyright (C) 2009 Aliaksandr Stelmachonak
# Licensed under GNU General Public License v2

import os.path
import pygtk
pygtk.require('2.0')
import gobject, gtk
import pyDialer.wvdial.process as wprocess
import pyDialer.wvdial.config as wconfig
import pyDialer.guigtk.config as gconfig
import pyDialer.guigtk.utils as gutils
import pyDialer.guigtk.info as ginfo
from pyDialer.globs import *

HAS_NOTIFY = True
try:
  import pynotify
  if not pynotify.init("pyDialer"):
    HAS_NOTIFY = False
except ImportError:
  HAS_NOTIFY = False

def wvdial_detect_cmd():
  out = ''
  chosen_cmd = ''
  # if a wvdial path was provided in commandline, add it as first choice
  if ( pydialer_options['withWvdial'] ) and ( wvdial_std_cmds[0] != pydialer_options['withWvdial'] ):
    wvdial_std_cmds.insert(0, pydialer_options['withWvdial'])
  for cmd in wvdial_std_cmds:
    out = out + '\nchecking for ' + cmd + ' ... '
    if ( os.path.isfile(cmd) ):
      out = out + 'detected!'
      chosen_cmd = cmd
      break
    else:
      out = out + 'not available!'
  return (chosen_cmd, out)

def wvdial_detect_cfg():
  out = ''
  chosen_cfg = ''
  # if a wvdial path was provided in commandline, add it as first choice
  if ( pydialer_options['withConfig'] ) and ( wvdial_std_cfgs[0] != pydialer_options['withConfig'] ):
    wvdial_std_cfgs.insert(0, pydialer_options['withConfig'])
  for cfg in wvdial_std_cfgs:
    out = out + '\nchecking for ' + cfg + ' ... '
    if ( os.path.isfile(os.path.expanduser(cfg)) ):
      out = out + 'detected!'
      chosen_cfg = cfg
      break
    else:
      out = out + 'not available!'
  return (chosen_cfg, out)

class MainInterface:

  MSG_TYPE_SYSTEM = 0
  MSG_TYPE_HIGH = 1
  MSG_TYPE_CMD = 2
  MSG_TYPE_LOGO = 3
  CONN_STATE_DISCONNECTED = 0
  CONN_STATE_CONNECTING = 1
  CONN_STATE_CONNECTED = 2

  def __init__(self):
    self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
    self.icontheme = gtk.icon_theme_get_default()
    if ( self.icontheme.has_icon('modem') ):
      self.icon_modem = self.icontheme.load_icon('modem', gtk.icon_size_lookup(gtk.ICON_SIZE_BUTTON)[1], 0)
      self.icon_modem_small = self.icontheme.load_icon('modem', gtk.icon_size_lookup(gtk.ICON_SIZE_MENU)[1], 0)
    else:
      self.icon_modem = self.window.render_icon(gtk.STOCK_NETWORK, gtk.ICON_SIZE_BUTTON)
      self.icon_modem_small = self.window.render_icon(gtk.STOCK_NETWORK, gtk.ICON_SIZE_MENU)

    self.window.connect("delete-event", self.delev_cb)
    self.window.connect("destroy", self.destroy_cb)
    self.window.set_title("pyDialer")
    self.window.set_icon_from_file(gutils.get_file("modem-disconnected.png"))
    self.window.set_resizable(True)
    self.window.set_geometry_hints(self.window, -1, -1, -1, -1)
    self.window.set_gravity(gtk.gdk.GRAVITY_STATIC)
    
    self.statusicon = gtk.status_icon_new_from_file(gutils.get_file("modem-disconnected.png"))
    self.statusicon.set_tooltip('Disconnected')
    statusicon_menu = gtk.Menu()
    self.connectto_menu = gtk.Menu()
    self.statusicon_menu_dialup = gtk.ImageMenuItem('_Connect to')
    self.statusicon_menu_dialup.set_submenu(self.connectto_menu)
    self.statusicon_menu_dialup.set_image(gtk.image_new_from_pixbuf(self.icon_modem_small))
    self.statusicon_menu_shutdown = gtk.ImageMenuItem('_Disconnect')
    self.statusicon_menu_shutdown.set_image(gtk.image_new_from_pixbuf(self.icon_modem_small))
    self.statusicon_menu_shutdown.connect("activate", self.dial_cb)
    statusicon_menu_info = gtk.ImageMenuItem(gtk.STOCK_INFO)
    statusicon_menu_info.connect("activate", self.info_cb)
    self.statusicon_menu_show = gtk.ImageMenuItem('Sho_w')
    if ( self.icontheme.has_icon('window-new') ):
      self.statusicon_menu_show.set_image(gtk.image_new_from_icon_name('window-new', gtk.ICON_SIZE_MENU))
    self.statusicon_menu_show.connect_object("activate", self.toggle_window_visibility, True)
    self.statusicon_menu_hide = gtk.ImageMenuItem('_Hide')
    if ( self.icontheme.has_icon('window-close') ):
      self.statusicon_menu_hide.set_image(gtk.image_new_from_icon_name('window-close', gtk.ICON_SIZE_MENU))
    self.statusicon_menu_hide.connect_object("activate", self.toggle_window_visibility, False)
    statusicon_menu_quit = gtk.ImageMenuItem(gtk.STOCK_QUIT)
    statusicon_menu_quit.connect_object("activate", gtk.Widget.destroy, self.window)
    statusicon_menu.append(self.statusicon_menu_dialup)
    statusicon_menu.append(self.statusicon_menu_shutdown)
    statusicon_menu.append(statusicon_menu_info)
    statusicon_menu.append(self.statusicon_menu_show)
    statusicon_menu.append(self.statusicon_menu_hide)
    statusicon_menu.append(gtk.SeparatorMenuItem())
    statusicon_menu.append(statusicon_menu_quit)
    statusicon_menu.show_all()
    self.statusicon.connect("popup-menu", self.statusicon_cb, statusicon_menu)
    self.statusicon.connect_object("activate", self.toggle_window_visibility, None)
    self.statusicon_menu_show.hide()
    self.statusicon_menu_shutdown.hide()

    self.register_notifications()
    
    self.dial_button_obj = gutils.ImgLabelButtonObj(expandIcon=False, xAlignLabel=0.5)
    self.dial_button_obj.update_label('Connect')
    self.dial_button_obj.update_icon(pixbuf=self.icon_modem)
    self.dial_button = self.dial_button_obj.get_widget()
    self.dial_button.set_size_request(120, -1)
    self.dial_button.connect("clicked", self.dial_cb)
    self.dial_button.set_data('wpid', None)
    self.dial_button.set_data('wprc', None)
    self.dial_button.set_data('wcmd', None)
    self.dial_button.set_data('wcfg', None)
    self.dial_button.set_data('wsec', None)
    
    self.topbar_status_obj = gutils.ImgLabelObj(evBoxPadding=4)
    self.topbar_status_obj.update_label('Disconnected')
    self.topbar_status_obj.update_icon(stock_id=gtk.STOCK_DISCONNECT, stock_size=gtk.ICON_SIZE_MENU)
    self.topbar_status_obj.update_style('lightgray', '#092138')
    self.topbar_status_obj.get_widget().set_size_request(300, -1)

    self.topbar_profiles_bbox = gtk.HBox(False, 0)
    self.connections_combo_obj = gutils.ComboBoxObj()
    self.connections_combo = self.connections_combo_obj.get_widget()
    self.connections_combo.connect("changed", self.combochanged_cb)
    self.refresh_button = gtk.Button()
    self.refresh_button.set_image(gtk.image_new_from_stock(gtk.STOCK_REFRESH, gtk.ICON_SIZE_MENU))
    self.refresh_button.connect("clicked", self.refresh_cb)
    self.topbar_profiles_bbox.pack_start(self.connections_combo, False, False, 0)
    self.topbar_profiles_bbox.pack_start(self.refresh_button, False, False, 0)

    topbar_longbt_bbox = gtk.HButtonBox()
    topbar_longbt_bbox.set_layout(gtk.BUTTONBOX_END)
    topbar_imgbutton_config_obj = gutils.ImgLabelButtonObj()
    topbar_imgbutton_config_obj.update_label('config')
    topbar_imgbutton_config_obj.update_icon(stock_id=gtk.STOCK_PREFERENCES, stock_size=gtk.ICON_SIZE_MENU)
    topbar_imgbutton_config_obj.get_widget().connect("clicked", self.config_cb)
    self.topbar_imgbutton_details_obj = gutils.ImgLabelButtonObj()
    self.topbar_imgbutton_details_obj.update_label('details')
    self.topbar_imgbutton_details_obj.update_icon(stock_id=gtk.STOCK_GO_UP, stock_size=gtk.ICON_SIZE_MENU)
    self.topbar_imgbutton_details_obj.get_widget().connect("clicked", self.details_toggle_cb)
    topbar_imgbutton_info_obj = gutils.ImgLabelButtonObj()
    topbar_imgbutton_info_obj.update_label('info')
    topbar_imgbutton_info_obj.update_icon(stock_id=gtk.STOCK_INFO, stock_size=gtk.ICON_SIZE_MENU)
    topbar_imgbutton_info_obj.get_widget().connect("clicked", self.info_cb)
    topbar_longbt_bbox.add(topbar_imgbutton_config_obj.get_widget())
    topbar_longbt_bbox.add(topbar_imgbutton_info_obj.get_widget())
    topbar_longbt_bbox.add(self.topbar_imgbutton_details_obj.get_widget())
    
    topbar_sidebt_bbox = gtk.HBox(True, 0)
    topbar_sidebt_about = gtk.Button()
    topbar_sidebt_about.set_image(gtk.image_new_from_stock(gtk.STOCK_ABOUT, gtk.ICON_SIZE_MENU))
    topbar_sidebt_about.connect("clicked", self.about_cb)
    topbar_sidebt_quit = gtk.Button()
    topbar_sidebt_quit.set_image(gtk.image_new_from_stock(gtk.STOCK_QUIT, gtk.ICON_SIZE_MENU))
    topbar_sidebt_quit.connect_object("clicked", gtk.Widget.destroy, self.window)
    topbar_sidebt_bbox.pack_start(topbar_sidebt_about, False, False, 0)
    topbar_sidebt_bbox.pack_start(topbar_sidebt_quit, False, False, 0)
    
    topbar_bbar_hbox = gtk.HBox(False, 0)
    topbar_bbar_hbox.pack_start(self.topbar_profiles_bbox, False, True, 0)
    topbar_bbar_hbox.pack_start(topbar_longbt_bbox, True, True, 0)
    topbar_bbar_hbox.pack_start(topbar_sidebt_bbox, False, True, 0)
    
    self.details_textview = gtk.TextView()
    self.details_textview.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse('white'))
    self.details_textview.set_editable(False)
    self.details_textview.set_cursor_visible(False)
    self.details_textview.set_size_request(-1, 120)
    self.details_textbuf = self.details_textview.get_buffer()
    details_tagtable = self.details_textbuf.get_tag_table()
    details_systag = gtk.TextTag()
    details_systag.set_property('foreground', 'darkblue')
    details_cmdtag = gtk.TextTag()
    details_cmdtag.set_property('foreground', 'darkred')
    details_higtag = gtk.TextTag()
    details_higtag.set_property('foreground', 'darkgreen')
    details_logtag = gtk.TextTag()
    details_logtag.set_property('foreground', 'lightgray')
    details_logtag.set_property('background', '#092138')
    details_tagtable.add(details_systag)
    details_tagtable.add(details_cmdtag)
    details_tagtable.add(details_higtag)
    details_tagtable.add(details_logtag)
    self.details_tags = { MainInterface.MSG_TYPE_SYSTEM:details_systag , MainInterface.MSG_TYPE_HIGH:details_higtag , MainInterface.MSG_TYPE_CMD:details_cmdtag , MainInterface.MSG_TYPE_LOGO:details_logtag }
    self.details_textiter = self.details_textbuf.get_end_iter()
    self.details_mark = self.details_textbuf.create_mark(None, self.details_textiter, False)
    self.details_insert(' ~~ pyDialer ' + PYDIALER_VERSION + ' ~~ ', MainInterface.MSG_TYPE_LOGO)
    details_sw = gtk.ScrolledWindow()
    details_sw.add(self.details_textview)
    
    details_bvbox = gtk.VBox(False, 0)
    details_smallbt_clear = gtk.Button()
    details_smallbt_clear.set_image(gtk.image_new_from_stock(gtk.STOCK_CLEAR, gtk.ICON_SIZE_MENU))
    
    def details_smallbt_clear_cb(clear_bt, details_tv):
      details_tv.delete(details_tv.get_start_iter(), details_tv.get_end_iter())
      self.details_textiter = details_tv.get_end_iter()
    
    details_smallbt_clear.connect("clicked", details_smallbt_clear_cb, self.details_textbuf)
    details_bvbox.pack_start(details_smallbt_clear, False, True, 0)
    details_bvbox.pack_start(gtk.VBox(False, 0), True, True, 0)
    
    self.details_hbox = gtk.HBox(False, 2)
    self.details_hbox.pack_start(details_sw, True, True, 0)
    self.details_hbox.pack_start(details_bvbox, False, True, 0)
    self.vbox = gtk.VBox(False, 2)
    self.vbox.set_border_width(2)
    topbar_hbox = gtk.HBox(False, 2)
    topbar_vbox = gtk.VBox(False, 2)
    topbar_hbox.pack_start(self.dial_button, False, True, 0)
    topbar_hbox.pack_start(topbar_vbox, True, True, 0)
    topbar_vbox.pack_start(self.topbar_status_obj.get_widget(), False, False, 0)
    topbar_vbox.pack_start(gtk.HSeparator(), False, False, 0)
    topbar_vbox.pack_start(topbar_bbar_hbox, True, True, 0)
    self.vbox.pack_start(gtk.HSeparator(), False, True, 0)
    self.vbox.pack_start(topbar_hbox, False, True, 0)
    self.vbox.pack_start(gtk.HSeparator(), False, True, 0)
    self.vbox.pack_start(self.details_hbox, True, True, 0)
    self.window.add(self.vbox)
    self.window.hide_all()
    
    # handle some command line options
    if ( pydialer_options['hideDetails'] ):
      self.toggle_details_visibility(False)
    if ( pydialer_options['hideWindow'] ):
      self.toggle_window_visibility(False)
    else:
      self.window.show_all()
    # collect information about the main window
    self.window_size = self.window.get_size()
    self.window_position = self.window.get_position()
    # run the wvdial files detection
    self.detect_wvdial_files()
    # get information about conection sections
    self.refresh_cb(self.refresh_button)
    # start autodiling if required
    if ( pydialer_options['autoDial'] ) and ( self.dial_button.flags() & gtk.SENSITIVE ):
      self.connections_combo_obj.set_active_item(pydialer_options['autoDial'])
      self.dial_cb(self.dial_button)

  def register_notifications(self):
		pynotify.init("pyDialer")
		self.notify = pynotify.Notification("(empty)")
		self.notify.set_urgency(pynotify.URGENCY_LOW)
		#self.notify.attach_to_status_icon(self.statusicon)

  def info_notify(self, text, body=None, icon=None):
		self.notify.update("pyDialer", text, icon)
		self.notify.show()

  def destroy_cb(self, widget, data=None):
    wvd_prc = self.dial_button.get_data('wprc')
    if ( wvd_prc != None ):
      wvd_prc.stop()
    gtk.main_quit()

  def delev_cb(self, widget, event):
    self.toggle_window_visibility(False)
    return True # stop other events

  def dial_cb(self, widget):
    wvd_prc = self.dial_button.get_data('wprc')
    if ( wvd_prc == None ):
      cmd = self.dial_button.get_data('wcmd')
      cfg = self.dial_button.get_data('wcfg')
      sec = self.dial_button.get_data('wsec')
      eargs = pydialer_options['withParams']
      self.details_insert('\n\nlaunching ', MainInterface.MSG_TYPE_SYSTEM)
      self.details_insert(cmd + ' -C ' + cfg + ' ' + sec, MainInterface.MSG_TYPE_HIGH)
      if ( eargs ):
        self.details_insert(' ' + eargs, MainInterface.MSG_TYPE_HIGH)
      self.details_insert(' ...\n', MainInterface.MSG_TYPE_SYSTEM)
      wvd_prc = wprocess.WvDialProcess(cmd, cfg, sec, eargs, self.details_process_mt)
      wvd_prc.start()
      self.dial_button.set_data('wprc', wvd_prc)
      self.toggle_conn_state(MainInterface.CONN_STATE_CONNECTING)
    else:
      wvd_prc = self.dial_button.get_data('wprc')
      self.dial_button.set_data('wprc', None)
      self.details_insert('\ntermination requested, please wait...\n ', MainInterface.MSG_TYPE_SYSTEM)
      while gtk.events_pending():
        gtk.main_iteration(False)
      wvd_prc.stop()

  def connectto_cb(self, widget, section):
      self.connections_combo_obj.set_active_item(section)
      self.dial_cb(self.dial_button)

  def combochanged_cb(self, widget):
      self.dial_button.set_data('wsec', widget.get_active_text())

  def refresh_cb(self, widget):
    wcfg = self.dial_button.get_data('wcfg')
    self.connections_combo_obj.clear_items()
    # cleaning up context menu
    for child in self.connectto_menu.get_children():
      self.connectto_menu.remove(child)
    # trying to load sections from the config file
    try:
      config = wconfig.WvDialConfig(wcfg)
      config.load_from_file()
    except (wconfig.WvDialConfig.LoadFromFileError):
      self.dial_button.set_sensitive(False)
      msg_dlg = gtk.MessageDialog(parent=self.window, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
      type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE, message_format='No config file found.')
      msg_dlg.run()
      msg_dlg.destroy()
    except (wconfig.WvDialConfig.ParsingError):
      self.dial_button.set_sensitive(False)
      msg_dlg = gtk.MessageDialog(parent=self.window, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
      type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE, message_format='Invalid config file')
      msg_dlg.run()
      msg_dlg.destroy()
    except wconfig.WvDialConfig.ConflictingOptionsError, (section, opt1, opt2):
      self.dial_button.set_sensitive(False)
      msg_dlg = gtk.MessageDialog(parent=self.window, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
      type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE,
      message_format='Conflicting options in section \'%s\':\n\n\'%s\' and \'%s\'\n\nplease remove one of them.' % (section, opt1, opt2))
      msg_dlg.run()
      msg_dlg.destroy()
    else:
      self.dial_button.set_sensitive(True)
      # get all connection sections
      sections = config.get_sections()
      # update combobox
      self.connections_combo_obj.add_items(sections)
      # update context menu with new children
      for sect in sections:
        menuItem = gtk.MenuItem(sect)
        menuItem.connect("activate", self.connectto_cb, sect)
        self.connectto_menu.append(menuItem)
    finally:
      self.connectto_menu.append(gtk.SeparatorMenuItem())
      # add refresh item
      menuItem = gtk.ImageMenuItem('Refresh')
      menuItem.set_image(gtk.image_new_from_stock(gtk.STOCK_REFRESH, gtk.ICON_SIZE_MENU))
      menuItem.connect("activate", self.refresh_cb)
      self.connectto_menu.append(menuItem)
      # show updated menu
      self.connectto_menu.show_all()

  def info_cb(self, widget):
    ginfo.info_panel_show(self.window)
    self.info_update_cb()

  def info_update_cb(self):
    info_dat = self.get_info_from_wprc()
    ginfo.info_panel_update(info_dat)

  def config_cb(self, widget):
    wcfg = self.dial_button.get_data('wcfg')
    if ( wcfg != None ):
      gconfig.config_panel_show(self.window, wcfg)
    else:
      # no config file detected, ask user to create a config file
      if ( pydialer_options['withConfig'] ):
        config_filename = os.path.abspath(os.path.expanduser(pydialer_options['withConfig']))
      else:
        config_filename = os.path.abspath(os.path.expanduser('~/.wvdialrc'))
      yn_dlg = gtk.MessageDialog(parent=self.window, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                                 type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_YES_NO,
                                 message_format='No config file found.\nCreate config file\n%s ?' % config_filename)
      yn_dlg.set_default_response(gtk.RESPONSE_YES)
      response = yn_dlg.run()
      yn_dlg.destroy()
      if ( response == gtk.RESPONSE_YES ):
        config = wconfig.WvDialConfig(config_filename)
        config.empty_default()
        try:
          config.save_to_file()
        except wconfig.WvDialConfig.SaveToFileError:
          msg_dlg = gtk.MessageDialog(parent=self.window, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                                      type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE,
                                      message_format='Unable to create config file.\nPlease check file permissions.')
          msg_dlg.run()
          msg_dlg.destroy()
        else:
          # detect files again to get the new config file
          self.detect_wvdial_files(detect_cmd=False)

  def about_cb(self, widget):
    about_dialog = gtk.AboutDialog()
    about_dialog.set_transient_for(self.window)
    about_dialog.set_name('pyDialer')
    about_dialog.set_version(PYDIALER_VERSION)
    about_dialog.set_comments('a simple and handy frontend for WvDial written in Python, fork of pyWvDial')
    about_dialog.set_copyright('copyright 2008 Giacomo Lozito\ncopyright 2009 Aiaksandr Stelmachonak')
    about_dialog.set_authors(['Giacomo Lozito <james@develia.org>\nAliaksandr Stelmachonak <mail.avatar@gmail.com>'])
    about_dialog.set_license('''
      Copyright (C) 2008 Giacomo Lozito
      Copyright (C) 2009 Aiaksandr Stelmachonak
      Licensed under GNU General Public License v2\n
      This program is free software; you can redistribute it and/or
      modify it under the terms of the GNU General Public License
      as published by the Free Software Foundation; either version 2
      of the License, or (at your option) any later version.\n
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.\n
      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.''')
    about_dialog.set_website('http://code.google.com/p/pydialer')
    about_dialog.set_logo(self.icon_modem)
    def about_cb_close(dialog, response_id):
      dialog.destroy()
    about_dialog.connect("response", about_cb_close)
    about_dialog.show_all()

  def statusicon_cb(self, statusicon, button, activate_time, menu):
    menu.popup(None, None, gtk.status_icon_position_menu, button, activate_time, statusicon)

  def get_info_from_wprc(self):
    wvd_prc = self.dial_button.get_data('wprc')
    info_dat = { 'netif':None, 'lipaddr':None, 'ripaddr':None, 'pdnsaddr':None, 'sdnsaddr':None, 'conntime':None }
    if ( wvd_prc != None):
      info_dat['netif'] = wvd_prc.get_info(wprocess.WvDialProcess.INFO_NETIF)
      info_dat['lipaddr'] = wvd_prc.get_info(wprocess.WvDialProcess.INFO_LIPADDR)
      info_dat['ripaddr'] = wvd_prc.get_info(wprocess.WvDialProcess.INFO_RIPADDR)
      info_dat['pdnsaddr'] = wvd_prc.get_info(wprocess.WvDialProcess.INFO_PDNSADDR)
      info_dat['sdnsaddr'] = wvd_prc.get_info(wprocess.WvDialProcess.INFO_SDNSADDR)
      info_dat['conntime'] = wvd_prc.get_info(wprocess.WvDialProcess.INFO_CONNTIME)
    return info_dat

  def toggle_window_visibility(self, isVisible):
    if ( isVisible == None ):
      if ( self.window.flags() & gtk.VISIBLE ):
        isVisible = False
      else:
        isVisible = True
    if ( isVisible == False ):
      self.statusicon_menu_hide.hide()
      self.statusicon_menu_show.show()
      self.window_position = self.window.get_position()
      self.window.hide_all()
    else:
      self.statusicon_menu_show.hide()
      self.statusicon_menu_hide.show()
      self.window.move(self.window_position[0], self.window_position[1])
      self.window.show_all()

  def toggle_conn_state(self, state_code):
    wvd_sec = self.dial_button.get_data('wsec')
    if ( state_code == MainInterface.CONN_STATE_DISCONNECTED ): # disconnected
      label = ' disconnected'
      status_icon = gutils.get_file("modem-disconnected.png")
      self.topbar_profiles_bbox.set_sensitive(True)
      self.dial_button.set_data('wprc', None)
      self.dial_button_obj.update_label('Connect')
      self.dial_button_obj.update_icon(pixbuf=self.icon_modem)
      self.statusicon_menu_shutdown.hide()
      self.statusicon_menu_dialup.show()
      self.topbar_status_obj.update_label(label)
      self.topbar_status_obj.update_icon(stock_id=gtk.STOCK_DISCONNECT, stock_size=gtk.ICON_SIZE_MENU)
      self.details_insert('\nok: ', MainInterface.MSG_TYPE_HIGH)
      self.details_insert('program terminated\n ', MainInterface.MSG_TYPE_SYSTEM)
      self.info_update_cb()
    elif ( state_code == MainInterface.CONN_STATE_CONNECTING ): # connecting
      label = ' connecting...'
      status_icon = gutils.get_file("modem-dialing.png")
      self.dial_button_obj.update_label('Disconnect')
      self.dial_button_obj.update_icon(pixbuf=self.icon_modem)
      self.statusicon_menu_dialup.hide()
      self.statusicon_menu_shutdown.show()
      self.topbar_profiles_bbox.set_sensitive(False)
      self.topbar_status_obj.update_label(label)
    elif ( state_code == MainInterface.CONN_STATE_CONNECTED ): # connected
      wvd_prc = self.dial_button.get_data('wprc')
      netif = wvd_prc.get_info(wprocess.WvDialProcess.INFO_NETIF)
      lipaddr = wvd_prc.get_info(wprocess.WvDialProcess.INFO_LIPADDR)
      label = ' connected'
      status_icon = gutils.get_file("modem-connected.png")
      extended_label = wvd_sec + label
      if ( netif ): extended_label = extended_label + ' on ' + netif
      if ( lipaddr ): extended_label = extended_label + ' (' + lipaddr + ')'
      self.topbar_status_obj.update_label(extended_label)
      self.topbar_status_obj.update_icon(stock_id=gtk.STOCK_CONNECT, stock_size=gtk.ICON_SIZE_MENU)
      self.topbar_profiles_bbox.set_sensitive(False)
      self.info_update_cb()
    self.window.set_icon_from_file(status_icon)
    if HAS_NOTIFY: self.info_notify(wvd_sec + label, icon = status_icon)
    self.statusicon.set_from_file(status_icon)
    self.statusicon.set_tooltip(wvd_sec + label)
    
  def toggle_details_visibility(self, isVisible):
    if ( isVisible == None ):
      if ( self.details_hbox.flags() & gtk.VISIBLE ):
        isVisible = False
      else:
        isVisible = True
    if ( isVisible == False ):
      self.details_hbox.hide_all()
      self.details_hbox.set_no_show_all(True)
      self.window_size = self.window.get_size()
      self.window.resize(self.window_size[0], 1)
      self.topbar_imgbutton_details_obj.update_icon(stock_id=gtk.STOCK_GO_DOWN, stock_size=gtk.ICON_SIZE_MENU)
      self.window.set_geometry_hints(self.window, -1, -1, 2000, 1) # could use sys.maxint in place of 2000
    else:
      self.details_hbox.set_no_show_all(False)
      self.details_hbox.show_all()
      self.window.resize(self.window.get_size()[0], self.window_size[1])
      self.topbar_imgbutton_details_obj.update_icon(stock_id=gtk.STOCK_GO_UP, stock_size=gtk.ICON_SIZE_MENU)
      self.window.set_geometry_hints(self.window, -1, -1, -1, -1)

  def details_toggle_cb(self, widget):
    if ( self.details_hbox.flags() & gtk.VISIBLE ):
      self.toggle_details_visibility(False)
    else:
      self.toggle_details_visibility(True)

  def details_insert(self, message, message_type):
    if ( message_type < 0 ) or ( message_type > 3 ):
      message_type = MainInterface.MSG_TYPE_SYSTEM
    self.details_textbuf.insert_with_tags(self.details_textiter, message, self.details_tags[message_type])
    self.details_textview.scroll_to_mark(self.details_mark, 0.0)

  def details_process_mt(self, data, code):
    # thread safe function that is called from another thread in WvDialProcess
    gobject.idle_add(self.details_process, data ,code)

  def details_process(self, data, code):
    gtk.gdk.threads_enter()
    if (code == wprocess.WvDialProcess.ST_INTERFACE_MSG):
      # print message in details textview
      self.details_insert(data, MainInterface.MSG_TYPE_CMD)
    elif (code == wprocess.WvDialProcess.ST_INTERFACE_CONN):
      # on conn signal, switch to state 'connected'
      self.toggle_conn_state(MainInterface.CONN_STATE_CONNECTED)
    elif (code == wprocess.WvDialProcess.ST_INTERFACE_HANG):
      # on hang signal, switch to state 'disconnected'
      self.toggle_conn_state(MainInterface.CONN_STATE_DISCONNECTED)
    elif (code == wprocess.WvDialProcess.ST_INTERFACE_PASSREQ):
      # on password request signal, ask user for a password
      wvd_prc = self.dial_button.get_data('wprc')
      if (wvd_prc != None ):
        passwd = self.ask_for_password()
        wvd_prc.send_password(passwd)
    gtk.gdk.threads_leave()

  def ask_for_password(self):
    pass_dialog = gtk.Dialog('pyDialer - Password Request', self.window, gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
    pass_dialog.set_default_response(gtk.RESPONSE_ACCEPT)
    label = gtk.Label('Please enter password for your ISP account:')
    entry = gtk.Entry()
    entry.set_activates_default(True)
    entry.set_visibility(False)
    def cbt_cb(bt, pe):
      if ( bt.get_active() == True ):
        pe.set_visibility(True)
      else:
        pe.set_visibility(False)
    cbt = gtk.CheckButton('show typed password')
    cbt.connect("toggled", cbt_cb, entry)
    vbox = gtk.VBox(False, 2)
    vbox.set_border_width(4)
    vbox.pack_start(label, True, True, 0)
    vbox.pack_start(entry, True, True, 0)
    vbox.pack_start(cbt, True, True, 0)
    frame = gtk.Frame()
    frame.set_border_width(4)
    frame.add(vbox)
    pass_dialog.vbox.pack_start(frame, True, True, 0)
    pass_dialog.show_all()
    response = pass_dialog.run()
    if ( response == gtk.RESPONSE_ACCEPT ):
      passwd = entry.get_text()
    else:
      passwd = ''
    pass_dialog.destroy()
    return passwd

  def detect_wvdial_files(self, detect_cmd=True, detect_cfg=True):
    if ( detect_cmd ):
      dcmd_res = wvdial_detect_cmd()
      self.details_insert(dcmd_res[1], MainInterface.MSG_TYPE_SYSTEM)
      if ( dcmd_res[0] == '' ): # no valid executable found
        self.dial_button.set_sensitive(False)
        self.details_insert('\nerror: ', MainInterface.MSG_TYPE_HIGH)
        self.details_insert('unable to find any valid wvdial executable!', MainInterface.MSG_TYPE_SYSTEM)
        return
      else: # attach wvdial filename to dial button
        self.dial_button.set_data('wcmd', dcmd_res[0])
    if ( detect_cfg) :
      dcfg_res = wvdial_detect_cfg()
      self.details_insert(dcfg_res[1], MainInterface.MSG_TYPE_SYSTEM)
      if ( dcfg_res[0] == '' ): # no valid config found
        self.dial_button.set_sensitive(False)
        self.details_insert('\nerror: ', MainInterface.MSG_TYPE_HIGH)
        self.details_insert('unable to find any valid config!', MainInterface.MSG_TYPE_SYSTEM)
        return
      else: # attach config filename to dial button
        self.dial_button.set_data('wcfg', dcfg_res[0])
    if ( self.dial_button.get_data('wcmd') ) and ( self.dial_button.get_data('wcfg') ):
      self.details_insert('\nok: ', MainInterface.MSG_TYPE_HIGH)
      self.details_insert('environment set up!', MainInterface.MSG_TYPE_SYSTEM)
      self.dial_button.set_sensitive(True)

  def main(self):
    gtk.main()
