#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# pyWvDial - "A simple and handy frontend for WvDial"
# Copyright (C) 2008 Giacomo Lozito
# Copyright (C) 2009 Aliaksandr Stelmachonak
# Licensed under GNU General Public License v2

import pygtk
pygtk.require('2.0')
import gobject, gtk
import os
import sys

"""Get path of resource files indepent of being installed or run in dist"""
def get_file(file):
    if os.path.exists(os.path.join(sys.prefix, "share", "pydialer", file)): return os.path.join(sys.prefix, "share", "pydialer", file)
    else: return os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..","data", file)

class PyGtkWrongVersion(Exception):
  pass



def check_pygtk_version():
  if (gtk.pygtk_version[0] < 2) or ((gtk.pygtk_version[0] == 2) and (gtk.pygtk_version[1] < 12)):
    raise PyGtkWrongVersion()



class ImgLabelObj:

  def __init__(self, expandLabel=True, expandIcon=False, putIconFirst=False, isLabelSelectable=True, isLabelEllipsizable=True, xAlignLabel=0.5, yAlignLabel=0.5, useEvbox=True, evBoxPadding=0):
    self.useEvbox = useEvbox
    self.label = gtk.Label()
    self.label.set_alignment(xAlignLabel, yAlignLabel)
    self.label.set_selectable(isLabelSelectable)
    self.img = gtk.Image()
    self.hbox = gtk.HBox(False, 2)
    if ( putIconFirst == False ):
      self.hbox.pack_start(self.label, expandLabel, True, 0)
      self.hbox.pack_start(self.img, expandIcon, True, 0)
    else:
      self.hbox.pack_start(self.img, expandIcon, True, 0)
      self.hbox.pack_start(self.label, expandLabel, True, 0)
    if ( self.useEvbox == True ):
      self.evbox = gtk.EventBox()
      self.hbox.set_border_width(evBoxPadding)
      self.evbox.add(self.hbox)

  def update_label(self, text, useMarkup=False):
    if ( useMarkup == True ):
      self.label.set_markup(text)
    else:
      self.label.set_text(text)

  def update_style(self, fgcolor=None, bgcolor=None):
    if ( self.useEvbox == True ):
      if ( fgcolor != None ):
        self.label.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse(fgcolor))
      if ( bgcolor != None ):
        self.evbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(bgcolor))

  def update_icon(self, pixbuf=None, stock_id='', stock_size=gtk.ICON_SIZE_MENU):
    if (pixbuf != None):
      self.img.set_from_pixbuf(pixbuf) # icon from file path
    else:
      self.img.set_from_stock(stock_id, stock_size) # icon from stock name

  def get_widget(self):
    if ( self.useEvbox == True ):
      return self.evbox
    else:
      return self.hbox



class ImgLabelButtonObj (ImgLabelObj):

  def __init__(self, expandLabel=True, expandIcon=True, putIconFirst=True, isLabelSelectable=False, isLabelEllipsizable=False, xAlignLabel=0, yAlignLabel=0.5, useEvbox=False, evBoxPadding=0):
    ImgLabelObj.__init__(self, expandLabel, expandIcon, putIconFirst, isLabelSelectable, isLabelEllipsizable, xAlignLabel, yAlignLabel, useEvbox, evBoxPadding)
    self.button = gtk.Button()
    self.inside = self
    self.button.add(ImgLabelObj.get_widget(self))

  def get_widget(self):
    return self.button

class ComboBoxObj:
  def __init__(self, itemsList=[]):
    self.comboProfile = gtk.combo_box_new_text();
    self.itemsCount = 0
    self.items = []
    self.add_items(itemsList)

  def add_items(self, itemsList=[]):
    for item in itemsList:
      self.comboProfile.append_text(item)
    self.itemsCount += len(itemsList)
    self.items += itemsList
    if self.itemsCount > 0:
      self.comboProfile.set_active(0)

  def clear_items(self):
    for i in range(self.itemsCount):
      self.comboProfile.remove_text(0)
    self.items = []
    self.itemsCount = 0
  
  def set_active_item(self, item):
    try:
    	self.comboProfile.set_active(self.items.index(item))
    except ValueError:
    	self.comboProfile.set_active(self.items.index('Defaults'))
  def get_widget(self):
    return self.comboProfile

